!******************************************************************************!
!                                                                              !
!    lcUUVa.f90                                                                !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcUUVa.f90                                     |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2016                                 |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2018-04-01 12:00             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcUUVa
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec
  contains
    subroutine loopUUVa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,      &
    riMaxOut, rankInc)
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(UUaV0)
!       P(3,2)*c0
        do j = riMaxIn, 0, -1
          wp(:,j) = P2(:)*cima*co(0)*w2(0)*wl(0,j)
        end do 
      case(UaVU0)
!       P(2,1)*c0
        do j = riMaxIn, 0, -1
          x0 = cima*co(0)*wl(0,j)
          wp(0,j) = -P1(0)*w2(0)*x0 + P1(1)*w2(1)*x0 + P1(2)*w2(2)*x0 + P1(3)* &
                    w2(3)*x0
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(0,riOut) = wp(0,riOut) - w2(mu)*x0
          end do 
        end do 
        wp(1:3,:) = cnul
      case(VUaU0)
!       P(1,2)*c0
        wp(0,:) = -P2(0)*cima*co(0)*w2(0)*wl(0,:) + P2(1)*cima*co(0)*w2(0)*wl(1&
                  ,:) + P2(2)*cima*co(0)*w2(0)*wl(2,:) + P2(3)*cima*co(0)*w2(0 &
                  )*wl(3,:)
        wp(1:3,:) = cnul
      case(VUUa0)
!       P(1,3)*c0
        x0 = cima*co(0)*w2(0)
        do j = riMaxIn, 0, -1
          wp(0,j) = P1(0)*wl(0,j)*x0 - P1(1)*wl(1,j)*x0 - P1(2)*wl(2,j)*x0 - P1&
                    (3)*wl(3,j)*x0 + P2(0)*wl(0,j)*x0 - P2(1)*wl(1,j)*x0 - P2(2&
                    )*wl(2,j)*x0 - P2(3)*wl(3,j)*x0
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(0,riOut) = wp(0,riOut) + wl(mu,j)*x0
          end do 
        end do 
        wp(1:3,:) = cnul
      case(UVUa0)
!       P(2,3)*c0
        do j = riMaxIn, 0, -1
          x0 = cima*co(0)*wl(0,j)
          wp(0,j) = P1(0)*w2(0)*x0 - P1(1)*w2(1)*x0 - P1(2)*w2(2)*x0 - P1(3)*w2&
                    (3)*x0 + P2(0)*w2(0)*x0 - P2(1)*w2(1)*x0 - P2(2)*w2(2)*x0  &
                     - P2(3)*w2(3)*x0
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(0,riOut) = wp(0,riOut) + w2(mu)*x0
          end do 
        end do 
        wp(1:3,:) = cnul
      case(UaUV0)
!       P(3,1)*c0
        do j = riMaxIn, 0, -1
          x0 = cima*co(0)*w2(0)*wl(0,j)
          wp(:,j) = P1(:)*x0
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + gg(:,mu)*x0
          end do 
        end do 
      end select 
    end subroutine loopUUVa
end module lcUUVa
